# -*- coding: utf-8 -*-
"""
@author:XuMing(xuming624@qq.com)
@description: 
"""

from nerpy.version import __version__

from nerpy.get_file import http_get, get_file
from nerpy.ner_model import NERModel
